@@ -1,34 +1,21 @@
-/* This file is part of the KDE libraries
-    Copyright 2008 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2008 David Faure <faure@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include <kiconloader.h>
 
+#include <QDir>
+#include <QRegularExpression>
 #include <QStandardPaths>
 #include <QTest>
-#include <QRegularExpression>
-#include <QTemporaryDir>
-#include <QGuiApplication>
 
-#include <kpixmapsequence.h>
+#include <KPixmapSequence>
 
-#include <KSharedConfig>
 #include <KConfigGroup>
+#include <KSharedConfig>
 
 extern KICONTHEMES_EXPORT void uintToHex(uint32_t colorData, QChar *buffer);
 
@@ -37,8 +24,9 @@ class KIconLoader_UnitTest : public QObject
     Q_OBJECT
 public:
     KIconLoader_UnitTest()
-        : testSizes({ 12, 22, 32, 42, 82, 132, 243 })
-    {}
+        : testSizes({12, 22, 32, 42, 82, 132, 243})
+    {
+    }
 
 private:
     QDir testDataDir;
@@ -53,7 +41,8 @@ private Q_SLOTS:
         QStandardPaths::setTestModeEnabled(true);
 
         const QStringList genericIconsFiles = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("mime/generic-icons"));
-        QVERIFY(!genericIconsFiles.isEmpty()); // KIconLoader relies on fallbacks to generic icons (e.g. x-office-document), which comes from a shared-mime-info file. Make sure it's installed!
+        QVERIFY(!genericIconsFiles.isEmpty()); // KIconLoader relies on fallbacks to generic icons (e.g. x-office-document), which comes from a shared-mime-info
+                                               // file. Make sure it's installed!
 
         KConfigGroup cg(KSharedConfig::openConfig(), "Icons");
         cg.writeEntry("Theme", "breeze");
@@ -66,8 +55,8 @@ private Q_SLOTS:
         appDataDir = QDir(testDataDir.absoluteFilePath(appName));
 
         // we will be recursively deleting these, so a sanity check is in order
-        QVERIFY(testIconsDir.absolutePath().contains(QStringLiteral("qttest")));
-        QVERIFY(appDataDir.absolutePath().contains(QStringLiteral("qttest")));
+        QVERIFY(testIconsDir.absolutePath().contains(QLatin1String("qttest")));
+        QVERIFY(appDataDir.absolutePath().contains(QLatin1String("qttest")));
 
         testIconsDir.removeRecursively();
         appDataDir.removeRecursively();
@@ -105,11 +94,14 @@ private Q_SLOTS:
 
         const QString breezeThemeFile = testIconsDir.filePath(QStringLiteral("breeze/index.theme"));
         QVERIFY(QFile::copy(QStringLiteral(":/breeze.theme"), breezeThemeFile));
-        //kde.png is missing, it should fallback to oxygen
-        //QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/apps/kde.png"))));
-        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoContext/iconindirectorywithoutcontext.png"))));
-        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoType/iconindirectorywithouttype.png"))));
-        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoContextOrType/iconindirectorywithoutcontextortype.png"))));
+        // kde.png is missing, it should fallback to oxygen
+        // QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/apps/kde.png"))));
+        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"),
+                            testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoContext/iconindirectorywithoutcontext.png"))));
+        QVERIFY(
+            QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoType/iconindirectorywithouttype.png"))));
+        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"),
+                            testIconsDir.filePath(QStringLiteral("breeze/22x22/appsNoContextOrType/iconindirectorywithoutcontextortype.png"))));
         QVERIFY(QFile::copy(QStringLiteral(":/anim-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/animations/process-working.png"))));
         QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/mimetypes/text-plain.png"))));
         QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/mimetypes/application-octet-stream.png"))));
@@ -120,19 +112,25 @@ private Q_SLOTS:
         QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/mimetypes/unknown.png"))));
         QVERIFY(QFile::copy(QStringLiteral(":/coloredsvgicon.svg"), testIconsDir.filePath(QStringLiteral("breeze/22x22/apps/coloredsvgicon.svg"))));
 
-        QVERIFY(QFile::setPermissions(breezeThemeFile, QFileDevice::ReadOwner|QFileDevice::WriteOwner));
+        // prepare some icons for our actions test
+        // when querying breeze for 'one-two', we expect
+        // 'one' from breeze instead of oxygen's 'one-two'.
+        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("oxygen/22x22/actions/one-two.png"))));
+        QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), testIconsDir.filePath(QStringLiteral("breeze/22x22/actions/one.png"))));
+
+        QVERIFY(QFile::setPermissions(breezeThemeFile, QFileDevice::ReadOwner | QFileDevice::WriteOwner));
         KConfig configFile(breezeThemeFile);
         KConfigGroup iconThemeGroup = configFile.group("Icon Theme");
         QVERIFY(iconThemeGroup.isValid());
         QStringList dirs = iconThemeGroup.readEntry("Directories", QStringList());
-        Q_FOREACH(int i, testSizes) {
+        for (int i : testSizes) {
             const QString relDir = QStringLiteral("%1x%1/emblems").arg(i);
             const QString dir = testIconsDir.filePath(QStringLiteral("breeze/") + relDir);
             QVERIFY(QDir().mkpath(dir));
 
             QPixmap img(i, i);
             img.fill(Qt::red);
-            QVERIFY(img.save(dir + "/red.png"));
+            QVERIFY(img.save(dir + QStringLiteral("/red.png")));
 
             dirs += relDir;
             KConfigGroup dirGroup = configFile.group(relDir);
@@ -157,7 +155,7 @@ private Q_SLOTS:
     void init()
     {
         // Remove icon cache
-        const QString cacheFile = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + "/icon-cache.kcache";
+        const QString cacheFile = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QStringLiteral("/icon-cache.kcache");
         QFile::remove(cacheFile);
 
         // Clear SHM cache
@@ -177,26 +175,22 @@ private Q_SLOTS:
         QString actionIconsDir = testIconsDir.filePath(actionIconsSubdir);
 
         QString nonExistingIconName = QStringLiteral("fhqwhgads_homsar");
-        QString newIconPath = actionIconsDir + QLatin1String("/")
-                              + nonExistingIconName + QLatin1String(".png");
+        QString newIconPath = actionIconsDir + QLatin1String("/") + nonExistingIconName + QLatin1String(".png");
         QFile::remove(newIconPath);
 
         KIconLoader iconLoader;
 
         // Find a non-existent icon, allowing unknown icon to be returned
-        QPixmap nonExistingIcon = iconLoader.loadIcon(
-                                      nonExistingIconName, KIconLoader::Toolbar);
+        QPixmap nonExistingIcon = iconLoader.loadIcon(nonExistingIconName, KIconLoader::Toolbar);
         QCOMPARE(nonExistingIcon.isNull(), false);
 
         // Install the existing icon by copying.
         QVERIFY(QFile::copy(QStringLiteral(":/test-22x22.png"), newIconPath));
 
         // Verify the icon can now be found.
-        QPixmap nowExistingIcon = iconLoader.loadIcon(
-                                      nonExistingIconName, KIconLoader::Toolbar);
+        QPixmap nowExistingIcon = iconLoader.loadIcon(nonExistingIconName, KIconLoader::Toolbar);
         QVERIFY(nowExistingIcon.cacheKey() != nonExistingIcon.cacheKey());
-        QCOMPARE(iconLoader.iconPath(nonExistingIconName, KIconLoader::Toolbar),
-                 newIconPath);
+        QCOMPARE(iconLoader.iconPath(nonExistingIconName, KIconLoader::Toolbar), newIconPath);
     }
 
     void testLoadIconCanReturnNull()
@@ -206,7 +200,7 @@ private Q_SLOTS:
         // then with canReturnNull=true.
         KIconLoader iconLoader;
         // We expect a warning here... This doesn't work though, due to the extended debug
-        //QTest::ignoreMessage(QtWarningMsg, "KIconLoader::loadIcon: No such icon \"this-icon-does-not-exist\"");
+        // QTest::ignoreMessage(QtWarningMsg, "KIconLoader::loadIcon: No such icon \"this-icon-does-not-exist\"");
         QPixmap pix = iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16);
         QVERIFY(!pix.isNull());
         QCOMPARE(pix.size(), QSize(16, 16));
@@ -215,8 +209,8 @@ private Q_SLOTS:
         QVERIFY(!pix.isNull());
         QCOMPARE(pix.size(), QSize(16, 16));
         // And now set canReturnNull to true
-        pix = iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16, KIconLoader::DefaultState,
-                                  QStringList(), nullptr, true);
+        pix =
+            iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16, KIconLoader::DefaultState, QStringList(), nullptr, true);
         QVERIFY(pix.isNull());
         // Try getting the "unknown" icon again, to see if the above call didn't put a null icon into the cache...
         pix = iconLoader.loadIcon(QStringLiteral("this-icon-does-not-exist"), KIconLoader::Desktop, 16);
@@ -242,7 +236,7 @@ private Q_SLOTS:
     {
         KIconLoader appIconLoader(appName);
         QString iconPath = appIconLoader.iconPath(QStringLiteral("image1"), KIconLoader::User);
-        QCOMPARE(iconPath, appDataDir.filePath("pics/image1.png"));
+        QCOMPARE(iconPath, appDataDir.filePath(QStringLiteral("pics/image1.png")));
         QVERIFY(QFile::exists(iconPath));
 
         // Load it again, to use the "last loaded" cache
@@ -250,7 +244,7 @@ private Q_SLOTS:
         QCOMPARE(iconPath, iconPath2);
         // Load something else, to clear the "last loaded" cache
         QString iconPathTextEdit = appIconLoader.iconPath(QStringLiteral("image2"), KIconLoader::User);
-        QCOMPARE(iconPathTextEdit, appDataDir.filePath("pics/image2.png"));
+        QCOMPARE(iconPathTextEdit, appDataDir.filePath(QStringLiteral("pics/image2.png")));
         QVERIFY(QFile::exists(iconPathTextEdit));
         // Now load kdialog again, to use the real kiconcache
         iconPath2 = appIconLoader.iconPath(QStringLiteral("image1"), KIconLoader::User);
@@ -268,9 +262,10 @@ private Q_SLOTS:
             QPixmap pix = icon.pixmap(QSize(22, 22));
             QVERIFY(!pix.isNull());
         }
-        QCOMPARE(icon.actualSize(QSize(96, 22)), QSize(22, 22));
-        QCOMPARE(icon.actualSize(QSize(22, 96)), QSize(22, 22));
-        QCOMPARE(icon.actualSize(QSize(22, 16)), QSize(16, 16));
+        // Qt's implementation of actualSize does not crop to squares, ensure we don't do that either
+        QCOMPARE(icon.actualSize(QSize(96, 22)), QSize(96, 22));
+        QCOMPARE(icon.actualSize(QSize(22, 96)), QSize(22, 96));
+        QCOMPARE(icon.actualSize(QSize(22, 16)), QSize(22, 16));
 
         // Can we ask for a really small size?
         {
@@ -284,19 +279,19 @@ private Q_SLOTS:
         QTest::addColumn<QString>("iconName");
         QTest::addColumn<QString>("expectedFileName");
 
-        QTest::newRow("existing icon") << "text-plain" << "text-plain.png";
-        QTest::newRow("octet-stream icon") << "application-octet-stream" << "application-octet-stream.png";
-        QTest::newRow("non-existing icon") << "foo-bar" << "application-octet-stream.png";
+        QTest::newRow("existing icon") << QStringLiteral("text-plain") << QStringLiteral("text-plain.png");
+        QTest::newRow("octet-stream icon") << QStringLiteral("application-octet-stream") << QStringLiteral("application-octet-stream.png");
+        QTest::newRow("non-existing icon") << QStringLiteral("foo-bar") << QStringLiteral("application-octet-stream.png");
         // Test this again, because now we won't go into the "fast path" of loadMimeTypeIcon anymore.
-        QTest::newRow("existing icon again") << "text-plain" << "text-plain.png";
-        QTest::newRow("generic fallback") << "image-foo-bar" << "image-x-generic.png";
-        QTest::newRow("video generic fallback") << "video-foo-bar" << "video-x-generic.png";
-        QTest::newRow("image-x-generic itself") << "image-x-generic" << "image-x-generic.png";
-        QTest::newRow("x-office-document icon") << "x-office-document" << "x-office-document.png";
-        QTest::newRow("unavailable generic icon") << "application/x-font-vfont" << "application-octet-stream.png";
-        QTest::newRow("#184852") << "audio/x-tuxguitar" << "audio-x-generic.png";
-        QTest::newRow("#178847") << "image/x-compressed-xcf" << "image-x-generic.png";
-        QTest::newRow("mimetype generic icon") << "application-x-fluid" << "x-office-document.png";
+        QTest::newRow("existing icon again") << QStringLiteral("text-plain") << QStringLiteral("text-plain.png");
+        QTest::newRow("generic fallback") << QStringLiteral("image-foo-bar") << QStringLiteral("image-x-generic.png");
+        QTest::newRow("video generic fallback") << QStringLiteral("video-foo-bar") << QStringLiteral("video-x-generic.png");
+        QTest::newRow("image-x-generic itself") << QStringLiteral("image-x-generic") << QStringLiteral("image-x-generic.png");
+        QTest::newRow("x-office-document icon") << QStringLiteral("x-office-document") << QStringLiteral("x-office-document.png");
+        QTest::newRow("unavailable generic icon") << QStringLiteral("application/x-font-vfont") << QStringLiteral("application-octet-stream.png");
+        QTest::newRow("#184852") << QStringLiteral("audio/x-tuxguitar") << QStringLiteral("audio-x-generic.png");
+        QTest::newRow("#178847") << QStringLiteral("image/x-compressed-xcf") << QStringLiteral("image-x-generic.png");
+        QTest::newRow("mimetype generic icon") << QStringLiteral("application-x-fluid") << QStringLiteral("x-office-document.png");
     }
 
     void testLoadMimeTypeIcon()
@@ -305,19 +300,15 @@ private Q_SLOTS:
         QFETCH(QString, expectedFileName);
         KIconLoader iconLoader;
         QString path;
-        QPixmap pix = iconLoader.loadMimeTypeIcon(iconName, KIconLoader::Desktop, 24,
-                      KIconLoader::DefaultState, QStringList(),
-                      &path);
+        QPixmap pix = iconLoader.loadMimeTypeIcon(iconName, KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!pix.isNull());
-        QCOMPARE(path.section('/', -1), expectedFileName);
+        QCOMPARE(path.section(QLatin1Char('/'), -1), expectedFileName);
 
         // do the same test using a global iconloader, so that
         // we get into the final return statement, which can only happen
         // if d->extraDesktopIconsLoaded becomes true first....
         QString path2;
-        pix = KIconLoader::global()->loadMimeTypeIcon(iconName, KIconLoader::Desktop, 24,
-                KIconLoader::DefaultState, QStringList(),
-                &path2);
+        pix = KIconLoader::global()->loadMimeTypeIcon(iconName, KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path2);
         QVERIFY(!pix.isNull());
         QCOMPARE(path2, path);
     }
@@ -325,15 +316,15 @@ private Q_SLOTS:
     void testHasIcon()
     {
         // Do everything twice to check code paths that might use a cache
-        QVERIFY(KIconLoader::global()->hasIcon("kde"));
-        QVERIFY(KIconLoader::global()->hasIcon("kde"));
+        QVERIFY(KIconLoader::global()->hasIcon(QStringLiteral("kde")));
+        QVERIFY(KIconLoader::global()->hasIcon(QStringLiteral("kde")));
         KIconLoader iconLoader;
-        QVERIFY(iconLoader.hasIcon("kde"));
+        QVERIFY(iconLoader.hasIcon(QStringLiteral("kde")));
 
-        QVERIFY(KIconLoader::global()->hasIcon("process-working"));
-        QVERIFY(KIconLoader::global()->hasIcon("process-working"));
-        QVERIFY(!KIconLoader::global()->hasIcon("no-such-icon-exists"));
-        QVERIFY(!KIconLoader::global()->hasIcon("no-such-icon-exists"));
+        QVERIFY(KIconLoader::global()->hasIcon(QStringLiteral("process-working")));
+        QVERIFY(KIconLoader::global()->hasIcon(QStringLiteral("process-working")));
+        QVERIFY(!KIconLoader::global()->hasIcon(QStringLiteral("no-such-icon-exists")));
+        QVERIFY(!KIconLoader::global()->hasIcon(QStringLiteral("no-such-icon-exists")));
     }
 
     void testIconPath()
@@ -347,15 +338,32 @@ private Q_SLOTS:
         QVERIFY(QFile::exists(unknownPath));
     }
 
+    void testCorrectFallback()
+    {
+        // we want to prefer icons from the same theme
+
+        // so if we have something like:
+        /*
+            oxygen:
+                one-two
+
+            breeze:
+                one
+        */
+        // and we ask for 'one-two', we expect to see 'one' from breeze instead
+        // of 'one-two' from oxygen.
+        QString path;
+        KIconLoader::global()->loadIcon(QStringLiteral("one-two"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
+        QVERIFY(path.contains("breeze/22x22/actions"));
+    }
+
     void testPathStore()
     {
         QString path;
-        QPixmap pix = KIconLoader::global()->loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 0,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        QPixmap pix = KIconLoader::global()->loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 0, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
         QVERIFY(QFile::exists(path));
-        QVERIFY2(path.contains("32x32"), qPrintable(path));
+        QVERIFY2(path.contains(QLatin1String("32x32")), qPrintable(path));
         QCOMPARE(pix.size(), QSize(32, 32));
 
         // Compare with iconPath()
@@ -363,95 +371,77 @@ private Q_SLOTS:
         QCOMPARE(path2, path);
 
         // Now specify a size
-        pix = KIconLoader::global()->loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        pix = KIconLoader::global()->loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
         QVERIFY(QFile::exists(path));
-        QVERIFY2(path.contains("22x22"), qPrintable(path));
+        QVERIFY2(path.contains(QLatin1String("22x22")), qPrintable(path));
         QCOMPARE(pix.size(), QSize(24, 24));
 
-        QVERIFY(KIconLoader::global()->hasIcon("kde"));
+        QVERIFY(KIconLoader::global()->hasIcon(QStringLiteral("kde")));
 
         path = QString();
-        KIconLoader::global()->loadIcon(QStringLiteral("does_not_exist"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path, true /* canReturnNull */);
+        KIconLoader::global()
+            ->loadIcon(QStringLiteral("does_not_exist"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path, true /* canReturnNull */);
         QVERIFY2(path.isEmpty(), qPrintable(path));
 
         path = QStringLiteral("some filler to check loadIcon() clears the variable");
-        KIconLoader::global()->loadIcon(QStringLiteral("does_not_exist"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path, true /* canReturnNull */);
+        KIconLoader::global()
+            ->loadIcon(QStringLiteral("does_not_exist"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path, true /* canReturnNull */);
         QVERIFY2(path.isEmpty(), qPrintable(path));
 
-        //Test that addAppDir doesn't break loading of icons from the old known paths
+        // Test that addAppDir doesn't break loading of icons from the old known paths
         KIconLoader loader;
-        //only addAppDir
+        // only addAppDir
         loader.addAppDir(QStringLiteral("kiconloader_unittest"));
         path = QString();
-        loader.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
 
         path = QString();
-        loader.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
 
-        //only reconfigure
+        // only reconfigure
         KIconLoader loader2;
         path = QString();
         loader2.reconfigure(QStringLiteral("kiconloader_unittest"));
-        loader2.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader2.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
-        loader2.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader2.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
 
-        //both addAppDir and reconfigure
+        // both addAppDir and reconfigure
         KIconLoader loader3;
         path = QString();
         loader3.addAppDir(QStringLiteral("kiconloader_unittest"));
         loader3.reconfigure(QStringLiteral("kiconloader_unittest"));
-        loader3.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader3.loadIcon(QStringLiteral("kde"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
 
         path = QString();
-        loader3.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
+        loader3.loadIcon(QStringLiteral("image1"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
         QVERIFY(!path.isEmpty());
     }
 
-    void testPathsNoContextType() {
+    void testPathsNoContextType()
+    {
         {
             QString path;
-            KIconLoader::global()->loadIcon(QStringLiteral("iconindirectorywithoutcontext"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
-            QVERIFY(path.endsWith("appsNoContext/iconindirectorywithoutcontext.png"));
+            KIconLoader::global()
+                ->loadIcon(QStringLiteral("iconindirectorywithoutcontext"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
+            QVERIFY(path.endsWith(QLatin1String("appsNoContext/iconindirectorywithoutcontext.png")));
         }
         {
             QString path;
-            KIconLoader::global()->loadIcon(QStringLiteral("iconindirectorywithouttype"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
-            QVERIFY(path.endsWith("appsNoType/iconindirectorywithouttype.png"));
+            KIconLoader::global()
+                ->loadIcon(QStringLiteral("iconindirectorywithouttype"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
+            QVERIFY(path.endsWith(QLatin1String("appsNoType/iconindirectorywithouttype.png")));
         }
         {
             QString path;
-            KIconLoader::global()->loadIcon(QStringLiteral("iconindirectorywithoutcontextortype"), KIconLoader::Desktop, 24,
-                                        KIconLoader::DefaultState, QStringList(),
-                                        &path);
-            QVERIFY(path.endsWith("appsNoContextOrType/iconindirectorywithoutcontextortype.png"));
+            KIconLoader::global()
+                ->loadIcon(QStringLiteral("iconindirectorywithoutcontextortype"), KIconLoader::Desktop, 24, KIconLoader::DefaultState, QStringList(), &path);
+            QVERIFY(path.endsWith(QLatin1String("appsNoContextOrType/iconindirectorywithoutcontextortype.png")));
         }
     }
 
@@ -461,16 +451,24 @@ private Q_SLOTS:
         QVERIFY(!pix.isNull());
     }
 
+    void testUnknownIcon()
+    {
+        QPixmap pix = KIconLoader::unknown();
+        QPixmap pix2 = KIconLoader::unknown();
+        QCOMPARE(pix.cacheKey(), pix2.cacheKey());
+    }
+
     void testLoadPixmapSequence()
     {
-        KPixmapSequence seq =  KIconLoader::global()->loadPixmapSequence(QStringLiteral("process-working"), 22);
+        KPixmapSequence seq = KIconLoader::global()->loadPixmapSequence(QStringLiteral("process-working"), 22);
         QVERIFY(seq.isValid());
     }
 
-    void testAppropriateSizes() {
+    void testAppropriateSizes()
+    {
         const KIconLoader iconLoader;
         const QRegularExpression rx(QStringLiteral("/(\\d+)x\\d+/"));
-        for(int i=1; i<testSizes.last()*1.2; i+=3) {
+        for (int i = 1; i < testSizes.last() * 1.2; i += 3) {
             QString path;
             QPixmap pix = iconLoader.loadIcon(QStringLiteral("red"), KIconLoader::Desktop, i, KIconLoader::DefaultState, QStringList(), &path);
             QVERIFY(!path.isEmpty());
@@ -499,9 +497,9 @@ private Q_SLOTS:
         qApp->setPalette(pal);
         QImage img = KIconLoader::global()->loadIcon(QStringLiteral("coloredsvgicon"), KIconLoader::NoGroup).toImage();
         QVERIFY(!img.isNull());
-        //Has the image been recolored to red,
-        //that is the color we wrote in kdeglobals as text color?
-        QCOMPARE(img.pixel(0,0), (uint)4294901760);
+        // Has the image been recolored to red,
+        // that is the color we wrote in kdeglobals as text color?
+        QCOMPARE(img.pixel(0, 0), (uint)4294901760);
     }
 
     void testUintToHex()
@@ -514,14 +512,28 @@ private Q_SLOTS:
 
         // Verify the ARGB hex (ff6496c8)
         uintToHex(testColorNoAlpha.rgba(), argbHex.data());
-        QCOMPARE(argbHex, QString("ff6496c8"));
+        QCOMPARE(argbHex, QStringLiteral("ff6496c8"));
 
         // HEX (ARGB format without the #): 7b6496c8
         QColor testColorWithAlpha(100, 150, 200, 123);
 
         // Test uintToHex to verify its ARGB output.
         uintToHex(testColorWithAlpha.rgba(), argbHex.data());
-        QCOMPARE(argbHex, QString("7b6496c8"));
+        QCOMPARE(argbHex, QStringLiteral("7b6496c8"));
+    }
+
+    void testQDirSetSearchPaths()
+    {
+        // setup search path for testprefix: => we shall find the iconinspecialsearchpath.svg afterwards, see e.g. bug 434451
+        QDir::setSearchPaths("testprefix", QStringList(":/searchpathdefineddir"));
+        QPixmap pix = KIconLoader::global()->loadIcon(QStringLiteral("testprefix:iconinspecialsearchpath.svg"),
+                                                      KIconLoader::NoGroup,
+                                                      24,
+                                                      KIconLoader::DefaultState,
+                                                      QStringList(),
+                                                      nullptr,
+                                                      true);
+        QVERIFY(!pix.isNull());
     }
 };
 
